//******************************************
//* STM32RingBuffer.h
//* Adapted by: Fred Eady
//* Last Update: 08-28-2024
//* Notes:
//******************************************

#ifndef UARTRINGBUFFER_H_
#define UARTRINGBUFFER_H_

#include "stm32g0xx_hal.h"
#include <stdint.h>

// Set RingBuffer Size
#define ringBufSize 64

typedef struct
{
  uint8_t buffer[ringBufSize];
  volatile uint16_t head;
  volatile uint16_t tail;
} ringBuffer;

//********************************************************************
//* Initialize the RX and TX RingBuffers
//********************************************************************
void RingBuffer_Init(void);

//********************************************************************
//* Read a character from the RX RingBuffer
//********************************************************************
uint8_t readRxRing(void);

//********************************************************************
//* Store a character in the RX RingBuffer
//********************************************************************
void writeRxRing(uint8_t bite, ringBuffer *buffer);

//********************************************************************
//* Store a character in the TX RingBuffer
//********************************************************************
void writeTxRing(uint8_t txChar);

//********************************************************************
//* Check for characters in RX RingBuffer
//********************************************************************
uint16_t IsCharInQueue(void);

//********************************************************************
//* Reset the RX RingBuffer
//********************************************************************
void resetRxRing (void);

//********************************************************************
//* RingBuffer Interrupt Handler
//********************************************************************
void ringbuffer_IRQHandler (UART_HandleTypeDef *huart);


#endif /* UARTRINGBUFFER_H_ */
